library(ggplot2)
setwd("Q:/SeventhPlan/RPM - Model/Technical Appendix/")
midc <- read.csv("MidCForecast.csv", header=TRUE)

midc$Year <- c(0, rep(1:19, each=4), rep(20, 3))
 
#Per conversation with Steve focusing on the On-Peak High forecast
# compared to the Medium forecast for risk model
high.on.mod <- lm(log(High.On/Med.On)~Year+I(Year^2), data=midc)
#low.on.mod <- lm(log(Low.On/Med.On)~Year+I(Year^2), data=midc)
#high.off.mod <- lm(log(High.Off/Med.Off)~Year+I(Year^2), data=midc)
#low.off.mod <- lm(log(Low.Off/Med.Off)~Year+I(Year^2), data=midc)
high.on.coef <- high.on.mod$coef

#Per conversation with Massoud, using the fit for the upper forecast
coef.weight <- high.on.coef 

#Now trying to estimate seasonal factors using the history
hist <- read.csv("MidCHist.csv", header=TRUE, stringsAsFactors=FALSE)
hist$Date <- as.Date(hist$Date)
hist$Quarter <- floor((as.numeric(format(hist$Date, "%m"))-1)/3 + 1)
hist$Year <- format(hist$Date, "%Y")
qhist <- aggregate(MidC.On~Quarter+Year, data=hist, FUN=mean, na.rm=TRUE)
annual.hist <- aggregate(MidC.On~Year, data=hist, FUN=mean)
names(annual.hist) <- c("Year", "AnnPrice")
qhist <- merge(qhist, annual.hist)
qhist$Factor <- qhist$MidC.On/qhist$AnnPrice
qhistmean <- aggregate(Factor ~ Quarter, qhist, mean)
names(qhistmean) <- c("Quarter", "AvgQFactor")
qhist <- merge(qhist, qhistmean)
qhist$NormalizedFactor <- qhist$Factor/qhist$AvgQFactor 
qhist$LogNormalizedFactor <- log(qhist$NormalizedFactor)
#Excluding 2000 and 2001, these circumstances will be handled by the jump and it seems like the 
# seasonality would be distorted if these are included
qsd <- aggregate(LogNormalizedFactor ~ Quarter, data=subset(qhist, !Year%in%c(2000, 2001)), sd)


#What is the largest historic quarter jump as a log factor?
log(max(qhist$Factor))
log(min(qhist$Factor))

#According to Massoud we're trying to hit the high and low forecasts at 80% and 10% respectively
# for natural gas so using the same for Mid-C prices
generate.games <- function(x){
  year <- c(0, rep(1:19, each=4), rep(20, 3)) 
  (coef.weight/qnorm(.80)*rnorm(3))%*% t(matrix(c(rep(1,80), year, year^2), nrow=80, ncol=3))
}
games <- sapply(1:800, generate.games)
games.df <- as.data.frame(exp(games)*midc$Med.On)
games.df$period <- 1:80

long.game.df <- reshape(games.df, idvar="period", varying=list(1:800), direction="long")
game.plot <- ggplot(subset(long.game.df, time%in%1:10), aes(x=period, y=V1, group=time, colour="Simultation")) + geom_line()
game.plot <- game.plot + geom_line(data=midc, aes(x=Period, y=Low.On, group="LOW", colour="Low"), lwd=1)
game.plot <- game.plot + geom_line(data=midc, aes(x=Period, y=High.On, group="HIGH", colour="High"), lwd=1)
game.plot <- game.plot + ylab("Price ($/MMBTU)") + theme(legend.title=element_blank())
dev.new()
game.plot

sum(games.df[80, -c(801)] > midc$High.On[80])
sum(games.df[80, -c(801)] > midc$High.On[80])/800

#Here is seasonal SD beyond annual shapes as applied above
qsd$LogNormalizedFactor
generate.seasonal <- function(x, q.sds) {
  exp(rnorm(n=80, mean=rep(0, 4), sd=q.sds[c(4,1,2,3)]))
}
seasonal <- sapply(1:800, generate.seasonal, q.sds=qsd$LogNormalizedFactor)
games.df <- games.df[,-c(801)]*seasonal

long.game.df <- reshape(games.df, idvar="period", varying=list(1:800), direction="long")
game.plot <- ggplot(subset(long.game.df, time%in%1:10), aes(x=period, y=V1, group=time, colour="Simulation")) + geom_line()
game.plot <- game.plot + geom_line(data=midc, aes(x=Period, y=Low.On, group="LOW", colour="Low"), lwd=1)
game.plot <- game.plot + geom_line(data=midc, aes(x=Period, y=High.On, group="HIGH", colour="High"), lwd=1)
game.plot <- game.plot + ylab("Load (aMW)") + theme(legend.title=element_blank())
dev.new()
game.plot

#If all that looks good here are the coefficients
coef.weight

#Now to look at interdependence
load <- read.csv("WNLoadHist.csv", header=TRUE)
load.long <- reshape(load, idvar=c("Year", "Type"), varying=list(3:14), direction="long")
load.long$Quarter <- floor((load.long$time-1)/3 + 1)
qload <- aggregate(X1 ~ Year + Quarter, data=subset(load.long, Type=="Average"), FUN=mean)
names(qload) <- c("Year", "Quarter", "Load")

ngas <- read.csv("NGasHist.csv", header=TRUE, stringsAsFactors=FALSE)
ngas$Date <- as.Date(ngas$Date)
ngas$Quarter <- floor((as.numeric(format(ngas$Date, "%m"))-1)/3 + 1)
ngas$Year <- format(ngas$Date, "%Y")
qngas <- aggregate(Price ~ Year + Quarter, data=ngas, FUN=mean)
names(qngas) <- c("Year", "Quarter", "NGas")

hydro <- read.csv("HydroHist.csv", header=TRUE, stringsAsFactors=FALSE)
hydro$Date <- as.Date(hydro$Date)
hydro$Quarter <- floor((as.numeric(format(hydro$Date, "%m"))-1)/3 + 1)
hydro$Year <- format(hydro$Date, "%Y")
qhydro <- aggregate(Hydro ~ Year + Quarter, data=hydro, FUN=mean)

qall <- merge(qhist[,c("Year", "Quarter", "MidC.On")], qload)
qall <- merge(qall, qngas)
qall <- merge(qall, qhydro)

inter.mod <- lm(log(MidC.On)~log(NGas)+Hydro+Load+0, data=qall)
inter.mod$coef
